@@ -710,6 +710,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                             cfg[action.dest] = str(cfg[action.dest])
                 elif isinstance(action, ActionParser):
                     cleanup_actions(cfg, action._parser._actions)
+                elif isinstance(action, ActionJsonSchema) and action._annotation is not None and cfg.get(action.dest) is not None:
+                    cfg[action.dest] = ActionJsonSchema._adapt_types(cfg[action.dest], action._annotation, action._subschemas, True)
 
         def cleanup_types(cfg):
             for dest, val in cfg.items():
@@ -728,7 +730,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         cfg = namespace_to_dict(_dict_to_flat_namespace(cfg))
         cleanup_actions(cfg, self._actions)
         cleanup_types(cfg)
-        cfg = _flat_namespace_to_dict(dict_to_namespace(cfg))
+        cfg = _flat_namespace_to_dict(_dict_to_flat_namespace(cfg))
 
         if format == 'parser_mode':
             format = 'yaml' if self.parser_mode == 'yaml' else 'json_indented'
